@@ -121,15 +121,19 @@ struct
            then check_seq_cons pref cs1 cs2' seq_sut' (c2::seq_trace)
            else (Spec.cleanup seq_sut'; false))
 
+    (* Common magic constants *)
+    let seq_len = 20   (* max length of the sequential prefix *)
+    let par_len = 12   (* max length of the parallel cmd lists *)
+
     (* Linearization test *)
     let lin_test ~rep_count ~retries ~count ~name ~lin_prop =
-      let arb_cmd_triple = arb_cmds_triple 20 12 in
+      let arb_cmd_triple = arb_cmds_triple seq_len par_len in
       Test.make ~count ~retries ~name
         arb_cmd_triple (repeat rep_count lin_prop)
 
     (* Negative linearization test *)
     let neg_lin_test ~rep_count ~retries ~count ~name ~lin_prop =
-      let arb_cmd_triple = arb_cmds_triple 20 12 in
+      let arb_cmd_triple = arb_cmds_triple seq_len par_len in
       Test.make_neg ~count ~retries ~name
         arb_cmd_triple (repeat rep_count lin_prop)
   end
@@ -137,11 +141,11 @@ end
 
 (* Type-representing values *)
 
-type constructible = |
-type deconstructible = |
+type constructible = private Constructible [@@ocaml.warning "-37"]
+type deconstructible = private Deconstructible [@@ocaml.warning "-37"]
 
-type combinable
-type noncombinable
+type combinable = private Combinable [@@ocaml.warning "-37"]
+type noncombinable = private Noncombinable [@@ocaml.warning "-37"]
 
 type (_,_,_,_) ty =
   | Gen : 'a QCheck.arbitrary * ('a -> string) -> ('a, constructible, 's, combinable) ty
@@ -155,7 +159,18 @@ let gen_deconstructible gen print eq = GenDeconstr (gen,print,eq)
 
 let qcheck_nat64_small = QCheck.(map Int64.of_int small_nat)
 
-let bytes_small_printable = QCheck.bytes_small_of QCheck.Gen.printable
+(* QCheck's string shrinker reduces each char repeatedly which is too excessive for Lin *)
+let shrink_char c = QCheck.(if c = 'a' then Iter.empty else Iter.return 'a')
+let shrink_string = QCheck.Shrink.string ~shrink:shrink_char
+let shrink_bytes = QCheck.Shrink.bytes ~shrink:shrink_char
+
+let string = QCheck.(set_shrink shrink_string string)
+let string_small = QCheck.(set_shrink shrink_string small_string)
+let string_small_printable = QCheck.(set_shrink shrink_string small_printable_string)
+
+let bytes = QCheck.(set_shrink shrink_bytes bytes)
+let bytes_small = QCheck.(set_shrink shrink_bytes bytes_small)
+let bytes_small_printable = QCheck.(set_shrink shrink_bytes (bytes_small_of Gen.printable))
 
 let unit =           GenDeconstr (QCheck.unit,           QCheck.Print.unit, (=))
 let bool =           GenDeconstr (QCheck.bool,           QCheck.Print.bool, (=))
@@ -166,15 +181,15 @@ let int =            GenDeconstr (QCheck.int,            QCheck.Print.int,  (=))
 let int_small =      GenDeconstr (QCheck.small_int,      QCheck.Print.int,  (=))
 let int_pos =        GenDeconstr (QCheck.pos_int,        QCheck.Print.int,  (=))
 let int_bound b =    GenDeconstr (QCheck.int_bound b,    QCheck.Print.int,  (=))
-let int32 =          GenDeconstr (QCheck.int32,          Int32.to_string,   Int32.equal)
-let int64 =          GenDeconstr (QCheck.int64,          Int64.to_string,   Int64.equal)
-let nat64_small =    GenDeconstr (qcheck_nat64_small,    Int64.to_string,   Int64.equal)
+let int32 =          GenDeconstr (QCheck.int32,          QCheck.Print.int32,   Int32.equal)
+let int64 =          GenDeconstr (QCheck.int64,          QCheck.Print.int64,   Int64.equal)
+let nat64_small =    GenDeconstr (qcheck_nat64_small,    QCheck.Print.int64,   Int64.equal)
 let float =          GenDeconstr (QCheck.float,          QCheck.Print.float,  Float.equal)
-let string =         GenDeconstr (QCheck.string,         QCheck.Print.string, String.equal)
-let string_small =   GenDeconstr (QCheck.small_string,   QCheck.Print.string, String.equal)
-let string_small_printable = GenDeconstr (QCheck.small_printable_string, QCheck.Print.string, String.equal)
-let bytes =          GenDeconstr (QCheck.bytes,          QCheck.Print.bytes, Bytes.equal)
-let bytes_small =    GenDeconstr (QCheck.bytes_small,    QCheck.Print.bytes, Bytes.equal)
+let string =         GenDeconstr (string,                QCheck.Print.string, String.equal)
+let string_small =   GenDeconstr (string_small,          QCheck.Print.string, String.equal)
+let string_small_printable = GenDeconstr (string_small_printable, QCheck.Print.string, String.equal)
+let bytes =          GenDeconstr (bytes,                 QCheck.Print.bytes, Bytes.equal)
+let bytes_small =    GenDeconstr (bytes_small,           QCheck.Print.bytes, Bytes.equal)
 let bytes_small_printable = GenDeconstr (bytes_small_printable, QCheck.Print.bytes, Bytes.equal)
 
 let option : type a c s. ?ratio:float -> (a, c, s, combinable) ty -> (a option, c, s, combinable) ty =
